@@ -34,6 +34,8 @@ from debian.deb822 import (
 
 from breezy.commit import PointlessCommit, NullCommitReporter
 from breezy.tree import Tree
+from breezy.workingtree import WorkingTree
+
 from debmutate.changelog import ChangelogEditor
 from debmutate.control import (
     ensure_relation,
@@ -78,12 +80,14 @@ from .build import (
     DetailedDebianBuildFailure,
     UnidentifiedDebianBuildError,
     )
+from ..logs import rotate_logfile
 from ..buildlog import problem_to_upstream_requirement
 from ..fix_build import BuildFixer, resolve_error
 from ..resolver.apt import (
     AptRequirement,
 )
-from .build import attempt_build, DEFAULT_BUILDER
+from .apt import AptManager
+from .build import attempt_build, DEFAULT_BUILDER, BUILD_LOG_FILENAME
 
 
 DEFAULT_MAX_ITERATIONS = 10
@@ -96,7 +100,7 @@ class CircularDependency(Exception):
         self.package = package
 
 
-class DebianPackagingContext(object):
+class DebianPackagingContext:
     def __init__(
         self, tree, subpath, committer, update_changelog, commit_reporter=None
     ):
@@ -109,7 +113,9 @@ class DebianPackagingContext(object):
     def abspath(self, *parts):
         return self.tree.abspath(os.path.join(self.subpath, *parts))
 
-    def commit(self, summary: str, update_changelog: Optional[bool] = None) -> bool:
+    def commit(
+            self, summary: str,
+            update_changelog: Optional[bool] = None) -> bool:
         if update_changelog is None:
             update_changelog = self.update_changelog
         with self.tree.lock_write():
@@ -151,12 +157,14 @@ class PackageDependencyFixer(BuildFixer):
         return req is not None
 
     def fix(self, error, phase):
-        reqs = problem_to_upstream_requirement(error)
-        if reqs is None:
+        req = problem_to_upstream_requirement(error)
+        if req is None:
             return False
 
-        if not isinstance(reqs, list):
-            reqs = [reqs]
+        if not isinstance(req, list):
+            reqs = [req]
+        else:
+            reqs = req
 
         changed = False
         for req in reqs:
@@ -187,24 +195,34 @@ def add_build_dependency(context, requirement: AptRequirement):
     if not isinstance(requirement, AptRequirement):
         raise TypeError(requirement)
 
-    control_path = context.abspath("debian/control")
+    if os.path.exists(context.abspath('debian/debcargo.toml')):
+        from debmutate.debcargo import DebcargoControlShimEditor
+        control = DebcargoControlShimEditor.from_debian_dir(
+            context.abspath('debian'))
+    else:
+        control_path = context.abspath("debian/control")
+        control = ControlEditor(path=control_path)
+
     try:
-        with ControlEditor(path=control_path) as updater:
+        with control as updater:
             for binary in updater.binaries:
                 if requirement.touches_package(binary["Package"]):
                     raise CircularDependency(binary["Package"])
             for rel in requirement.relations:
                 updater.source["Build-Depends"] = ensure_relation(
-                    updater.source.get("Build-Depends", ""), PkgRelation.str([rel])
+                    updater.source.get("Build-Depends", ""),
+                    PkgRelation.str([rel])
                 )
     except FormattingUnpreservable as e:
-        logging.info("Unable to edit %s in a way that preserves formatting.", e.path)
+        logging.info(
+            "Unable to edit %s in a way that preserves formatting.", e.path)
         return False
 
     desc = requirement.pkg_relation_str()
 
     if not updater.changed:
-        logging.info("Giving up; dependency %s was already present.", desc)
+        logging.info(
+            "Giving up; build dependency %s was already present.", desc)
         return False
 
     logging.info("Adding build dependency: %s", desc)
@@ -236,13 +254,18 @@ def add_test_dependency(context, testname, requirement):
                         control.get("Depends", ""), PkgRelation.str([rel])
                     )
     except FormattingUnpreservable as e:
-        logging.info("Unable to edit %s in a way that preserves formatting.", e.path)
-        return False
-    if not updater.changed:
+        logging.info(
+            "Unable to edit %s in a way that preserves formatting.", e.path)
         return False
 
     desc = requirement.pkg_relation_str()
 
+    if not updater.changed:
+        logging.info(
+            "Giving up; dependency %s for test %s was already present.",
+            desc, testname)
+        return False
+
     logging.info("Adding dependency to test %s: %s", testname, desc)
     return context.commit(
         "Add missing dependency for test %s on %s." % (testname, desc),
@@ -252,7 +275,8 @@ def add_test_dependency(context, testname, requirement):
 def targeted_python_versions(tree: Tree, subpath: str) -> List[str]:
     with tree.get_file(os.path.join(subpath, "debian/control")) as f:
         control = Deb822(f)
-    build_depends = PkgRelation.parse_relations(control.get("Build-Depends", ""))
+    build_depends = PkgRelation.parse_relations(
+        control.get("Build-Depends", ""))
     all_build_deps: Set[str] = set()
     for or_deps in build_depends:
         all_build_deps.update(or_dep["name"] for or_dep in or_deps)
@@ -276,9 +300,7 @@ def python_tie_breaker(tree, subpath, reqs):
             return True
         if pkg.startswith("lib%s-" % python_version):
             return True
-        if pkg == r'lib%s-dev' % python_version:
-            return True
-        return False
+        return pkg == r'lib%s-dev' % python_version
 
     for python_version in targeted:
         for req in reqs:
@@ -301,7 +323,8 @@ def retry_apt_failure(error, phase, apt, context):
 def enable_dh_autoreconf(context, phase):
     # Debhelper >= 10 depends on dh-autoreconf and enables autoreconf by
     # default.
-    debhelper_compat_version = get_debhelper_compat_level(context.tree.abspath("."))
+    debhelper_compat_version = get_debhelper_compat_level(
+        context.tree.abspath("."))
     if debhelper_compat_version is not None and debhelper_compat_version < 10:
 
         def add_with_autoreconf(line, target):
@@ -320,9 +343,8 @@ def enable_dh_autoreconf(context, phase):
 
 
 def fix_missing_configure(error, phase, context):
-    if not context.tree.has_filename("configure.ac") and not context.tree.has_filename(
-        "configure.in"
-    ):
+    if (not context.tree.has_filename("configure.ac")
+            and not context.tree.has_filename("configure.in")):
         return False
 
     return enable_dh_autoreconf(context, phase)
@@ -345,8 +367,8 @@ def fix_missing_config_status_input(error, phase, context):
         return False
 
     def add_autogen(mf):
-        rule = any(mf.iter_rules(b"override_dh_autoreconf"))
-        if rule:
+        rule_exists = any(mf.iter_rules(b"override_dh_autoreconf"))
+        if rule_exists:
             return
         rule = mf.add_rule(b"override_dh_autoreconf")
         rule.append_command(b"dh_autoreconf ./autogen.sh")
@@ -425,7 +447,8 @@ class SimpleBuildFixer(BuildFixer):
 
 
 class DependencyBuildFixer(BuildFixer):
-    def __init__(self, packaging_context, apt_resolver, problem_cls: Type[Problem], fn):
+    def __init__(self, packaging_context, apt_resolver,
+                 problem_cls: Type[Problem], fn):
         self.context = packaging_context
         self.apt_resolver = apt_resolver
         self._problem_cls = problem_cls
@@ -445,17 +468,20 @@ class DependencyBuildFixer(BuildFixer):
         return self._fn(problem, phase, self.apt_resolver, self.context)
 
 
-def versioned_package_fixers(session, packaging_context, apt):
+def versioned_package_fixers(session, packaging_context, apt: AptManager):
     return [
         PgBuildExtOutOfDateControlFixer(packaging_context, session, apt),
-        SimpleBuildFixer(packaging_context, MissingConfigure, fix_missing_configure),
+        SimpleBuildFixer(
+            packaging_context, MissingConfigure, fix_missing_configure),
         SimpleBuildFixer(
             packaging_context, MissingAutomakeInput, fix_missing_automake_input
         ),
         SimpleBuildFixer(
-            packaging_context, MissingConfigStatusInput, fix_missing_config_status_input
+            packaging_context, MissingConfigStatusInput,
+            fix_missing_config_status_input
         ),
-        SimpleBuildFixer(packaging_context, MissingPerlFile, fix_missing_makefile_pl),
+        SimpleBuildFixer(
+            packaging_context, MissingPerlFile, fix_missing_makefile_pl),
         SimpleBuildFixer(
             packaging_context, DebcargoUnacceptablePredicate,
             debcargo_coerce_unacceptable_prerelease),
@@ -465,17 +491,24 @@ def versioned_package_fixers(session, packaging_context, apt):
     ]
 
 
-def apt_fixers(apt, packaging_context) -> List[BuildFixer]:
+def apt_fixers(apt: AptManager, packaging_context,
+               dep_server_url: Optional[str] = None) -> List[BuildFixer]:
     from ..resolver.apt import AptResolver
     from .udd import popcon_tie_breaker
     from .build_deps import BuildDependencyTieBreaker
 
     apt_tie_breakers = [
-        partial(python_tie_breaker, packaging_context.tree, packaging_context.subpath),
+        partial(python_tie_breaker, packaging_context.tree,
+                packaging_context.subpath),
         BuildDependencyTieBreaker.from_session(apt.session),
         popcon_tie_breaker,
     ]
-    resolver = AptResolver(apt, apt_tie_breakers)
+    resolver: AptResolver
+    if dep_server_url:
+        from ..resolver.dep_server import DepServerAptResolver
+        resolver = DepServerAptResolver(apt, dep_server_url, apt_tie_breakers)
+    else:
+        resolver = AptResolver(apt, apt_tie_breakers)
     return [
         DependencyBuildFixer(
             packaging_context, apt, AptFetchFailure, retry_apt_failure
@@ -484,38 +517,46 @@ def apt_fixers(apt, packaging_context) -> List[BuildFixer]:
     ]
 
 
-def default_fixers(local_tree, subpath, apt, committer=None, update_changelog=None):
+def default_fixers(
+        local_tree: WorkingTree,
+        subpath: str, apt: AptManager,
+        committer: Optional[str] = None,
+        update_changelog: Optional[bool] = None,
+        dep_server_url: Optional[str] = None):
     packaging_context = DebianPackagingContext(
         local_tree, subpath, committer, update_changelog,
         commit_reporter=NullCommitReporter()
     )
-    return versioned_package_fixers(apt.session, packaging_context, apt) + apt_fixers(
-        apt, packaging_context
-    )
+    return (versioned_package_fixers(apt.session, packaging_context, apt)
+            + apt_fixers(apt, packaging_context, dep_server_url))
 
 
 def build_incrementally(
-    local_tree,
-    apt,
-    suffix,
-    build_suite,
-    output_directory,
-    build_command,
+    local_tree: WorkingTree,
+    apt: AptManager,
+    suffix: str,
+    build_suite: str,
+    output_directory: str,
+    build_command: str,
     build_changelog_entry,
-    committer=None,
-    max_iterations=DEFAULT_MAX_ITERATIONS,
-    subpath="",
+    committer: Optional[str] = None,
+    max_iterations: int = DEFAULT_MAX_ITERATIONS,
+    subpath: str = "",
     source_date_epoch=None,
-    update_changelog=True,
-    extra_repositories=None,
-    fixers=None,
+    update_changelog: bool = True,
+    apt_repository: Optional[str] = None,
+    apt_repository_key: Optional[str] = None,
+    extra_repositories: Optional[List[str]] = None,
+    fixers: Optional[List[BuildFixer]] = None,
     run_gbp_dch: Optional[bool] = None,
+    dep_server_url: Optional[str] = None,
 ):
     fixed_errors: List[Tuple[Problem, str]] = []
     if fixers is None:
         fixers = default_fixers(
             local_tree, subpath, apt, committer=committer,
-            update_changelog=update_changelog)
+            update_changelog=update_changelog,
+            dep_server_url=dep_server_url)
     logging.info("Using fixers: %r", fixers)
     if run_gbp_dch is None:
         run_gbp_dch = (update_changelog is False)
@@ -531,6 +572,8 @@ def build_incrementally(
                 subpath=subpath,
                 source_date_epoch=source_date_epoch,
                 run_gbp_dch=run_gbp_dch,
+                apt_repository=apt_repository,
+                apt_repository_key=apt_repository_key,
                 extra_repositories=extra_repositories,
             )
         except UnidentifiedDebianBuildError:
@@ -541,15 +584,19 @@ def build_incrementally(
                 logging.info("No relevant context, not making any changes.")
                 raise
             if (e.error, e.phase) in fixed_errors:
-                logging.warning("Error was still not fixed on second try. Giving up.")
+                logging.warning(
+                    "Error was still not fixed on second try. Giving up.")
                 raise
-            if max_iterations is not None and len(fixed_errors) > max_iterations:
-                logging.warning("Last fix did not address the issue. Giving up.")
+            if (max_iterations is not None
+                    and len(fixed_errors) > max_iterations):
+                logging.warning(
+                    "Last fix did not address the issue. Giving up.")
                 raise
             reset_tree(local_tree, subpath=subpath)
             try:
                 if not resolve_error(e.error, e.phase, fixers):
-                    logging.warning("Failed to resolve error %r. Giving up.", e.error)
+                    logging.warning(
+                        "Failed to resolve error %r. Giving up.", e.error)
                     raise
             except GeneratedFile:
                 logging.warning(
@@ -560,20 +607,13 @@ def build_incrementally(
                 raise e
             except CircularDependency:
                 logging.warning(
-                    "Unable to fix %r; it would introduce a circular " "dependency.",
+                    "Unable to fix %r; it would introduce a circular "
+                    "dependency.",
                     e.error,
                 )
                 raise e
             fixed_errors.append((e.error, e.phase))
-            if os.path.exists(os.path.join(output_directory, "build.log")):
-                i = 1
-                while os.path.exists(
-                    os.path.join(output_directory, "build.log.%d" % i)
-                ):
-                    i += 1
-                target_path = os.path.join(output_directory, "build.log.%d" % i)
-                os.rename(os.path.join(output_directory, "build.log"), target_path)
-                logging.debug("Storing build log at %s", target_path)
+            rotate_logfile(os.path.join(output_directory, BUILD_LOG_FILENAME))
 
 
 def main(argv=None):
@@ -582,13 +622,15 @@ def main(argv=None):
     parser = argparse.ArgumentParser("ognibuild.debian.fix_build")
     modifications = parser.add_argument_group('Modifications')
     modifications.add_argument(
-        "--suffix", type=str, help="Suffix to use for test builds.", default="fixbuild1"
+        "--suffix", type=str, help="Suffix to use for test builds.",
+        default="fixbuild1"
     )
     modifications.add_argument(
         "--suite", type=str, help="Suite to target.", default="unstable"
     )
     modifications.add_argument(
-        "--committer", type=str, help="Committer string (name and email)", default=None
+        "--committer", type=str, help="Committer string (name and email)",
+        default=None
     )
     modifications.add_argument(
         "--no-update-changelog",
@@ -621,13 +663,16 @@ def main(argv=None):
         default=DEFAULT_MAX_ITERATIONS,
         help='Maximum number of issues to attempt to fix before giving up.')
     build_behaviour.add_argument("--schroot", type=str, help="chroot to use.")
+    parser.add_argument(
+        "--dep-server-url", type=str,
+        help="ognibuild dep server to use",
+        default=os.environ.get('OGNIBUILD_DEPS'))
     parser.add_argument("--verbose", action="store_true", help="Be verbose")
 
     args = parser.parse_args()
-    from breezy.workingtree import WorkingTree
     import breezy.git  # noqa: F401
     import breezy.bzr  # noqa: F401
-    from .apt import AptManager
+    from ..session import Session
     from ..session.plain import PlainSession
     from ..session.schroot import SchrootSession
     import tempfile
@@ -646,9 +691,11 @@ def main(argv=None):
             output_directory = args.output_directory
             if not os.path.isdir(output_directory):
                 parser.error(
-                    'output directory %s is not a directory' % output_directory)
+                    'output directory %s is not a directory'
+                    % output_directory)
 
         tree = WorkingTree.open(".")
+        session: Session
         if args.schroot:
             session = SchrootSession(args.schroot)
         else:
@@ -670,6 +717,7 @@ def main(argv=None):
                 committer=args.committer,
                 update_changelog=args.update_changelog,
                 max_iterations=args.max_iterations,
+                dep_server_url=args.dep_server_url,
             )
         except DetailedDebianBuildFailure as e:
             if e.phase is None:
@@ -680,8 +728,10 @@ def main(argv=None):
                 phase = "%s (%s)" % (e.phase[0], e.phase[1])
             logging.fatal("Error during %s: %s", phase, e.error)
             if not args.output_directory:
-                xdg_cache_dir = os.environ.get('XDG_CACHE_HOME', os.path.expanduser('~/.cache'))
-                buildlogs_dir = os.path.join(xdg_cache_dir, 'ognibuild', 'buildlogs')
+                xdg_cache_dir = os.environ.get(
+                    'XDG_CACHE_HOME', os.path.expanduser('~/.cache'))
+                buildlogs_dir = os.path.join(
+                    xdg_cache_dir, 'ognibuild', 'buildlogs')
                 os.makedirs(buildlogs_dir, exist_ok=True)
                 target_log_file = os.path.join(
                     buildlogs_dir,
