@@ -26,16 +26,19 @@ from . import Requirement
 
 class PythonPackageRequirement(Requirement):
 
+    family = "python-package"
+
     package: str
 
-    def __init__(self, package, python_version=None, specs=None, minimum_version=None):
-        super(PythonPackageRequirement, self).__init__("python-package")
+    def __init__(
+            self, package, python_version=None, specs=None,
+            minimum_version=None):
         self.package = package
         self.python_version = python_version
-        if minimum_version is not None:
-            specs = [(">=", minimum_version)]
         if specs is None:
             specs = []
+        if minimum_version is not None:
+            specs.append((">=", minimum_version))
         self.specs = specs
 
     def __repr__(self):
@@ -53,11 +56,29 @@ class PythonPackageRequirement(Requirement):
             return "python package: %s" % (self.package,)
 
     @classmethod
-    def from_requirement_str(cls, text):
-        from requirements.requirement import Requirement
+    def from_requirement_str(cls, text, python_version=None):
+        from requirements.requirement import Requirement as RequirementEntry
 
-        req = Requirement.parse(text)
-        return cls(package=req.name, specs=req.specs)
+        req = RequirementEntry.parse(text)
+        return cls(
+            package=req.name, specs=req.specs, python_version=python_version)
+
+    def requirement_str(self):
+        if self.specs:
+            return '%s;%s' % (
+                self.package, ','.join([''.join(s) for s in self.specs]))
+        return self.package
+
+    @classmethod
+    def _from_json(cls, js):
+        if isinstance(js, str):
+            return cls.from_requirement_str(js)
+        return cls.from_requirement_str(js[0], python_version=js[1])
+
+    def _json(self):
+        if self.python_version:
+            return [self.requirement_str(), self.python_version]
+        return self.requirement_str()
 
     def met(self, session):
         if self.python_version == "cpython3":
@@ -74,7 +95,8 @@ class PythonPackageRequirement(Requirement):
             raise NotImplementedError
         text = self.package + ",".join(["".join(spec) for spec in self.specs])
         p = session.Popen(
-            [cmd, "-c", "import pkg_resources; pkg_resources.require(%r)" % text],
+            [cmd, "-c",
+             "import pkg_resources; pkg_resources.require(%r)" % text],
             stdout=subprocess.DEVNULL,
             stderr=subprocess.DEVNULL,
         )
@@ -82,16 +104,34 @@ class PythonPackageRequirement(Requirement):
         return p.returncode == 0
 
 
+Requirement.register_json(PythonPackageRequirement)
+
+
 class LatexPackageRequirement(Requirement):
 
+    family = "latex-package"
+
     def __init__(self, package: str):
         self.package = package
 
     def __repr__(self):
         return "%s(%r)" % (type(self).__name__, self.package)
 
+    def _json(self):
+        return self.package
+
+    @classmethod
+    def _from_json(cls, package):
+        return cls(package)
+
+
+Requirement.register_json(LatexPackageRequirement)
+
 
 class PhpPackageRequirement(Requirement):
+
+    family = "php-package"
+
     def __init__(
         self,
         package: str,
@@ -104,6 +144,13 @@ class PhpPackageRequirement(Requirement):
         self.min_version = min_version
         self.max_version = max_version
 
+    def _json(self):
+        return (self.package, self.channel, self.min_version, self.max_version)
+
+    @classmethod
+    def _from_json(cls, js):
+        return cls(*js)
+
     def __repr__(self):
         return "%s(%r, %r, %r, %r)" % (
             type(self).__name__,
@@ -114,14 +161,24 @@ class PhpPackageRequirement(Requirement):
         )
 
 
+Requirement.register_json(PhpPackageRequirement)
+
+
 class BinaryRequirement(Requirement):
 
+    family = "binary"
     binary_name: str
 
     def __init__(self, binary_name):
-        super(BinaryRequirement, self).__init__("binary")
         self.binary_name = binary_name
 
+    def _json(self):
+        return self.binary_name
+
+    @classmethod
+    def _from_json(cls, js):
+        return cls(js)
+
     def __repr__(self):
         return "%s(%r)" % (type(self).__name__, self.binary_name)
 
@@ -135,24 +192,40 @@ class BinaryRequirement(Requirement):
         return p.returncode == 0
 
 
+Requirement.register_json(BinaryRequirement)
+
+
 class PHPExtensionRequirement(Requirement):
 
+    family = "php-extension"
     extension: str
 
     def __init__(self, extension: str):
-        super(PHPExtensionRequirement, self).__init__("php-extension")
         self.extension = extension
 
     def __repr__(self):
         return "%s(%r)" % (type(self).__name__, self.extension)
 
 
+class PytestPluginRequirement(Requirement):
+
+    family = "pytest-plugin"
+
+    plugin: str
+
+    def __init__(self, plugin: str):
+        self.plugin = plugin
+
+    def __repr__(self):
+        return "%s(%r)" % (type(self).__name__, self.plugin)
+
+
 class VcsControlDirectoryAccessRequirement(Requirement):
 
     vcs: List[str]
+    family = "vcs-access"
 
     def __init__(self, vcs):
-        super(VcsControlDirectoryAccessRequirement, self).__init__("vcs-access")
         self.vcs = vcs
 
     def __repr__(self):
@@ -164,9 +237,9 @@ class PerlModuleRequirement(Requirement):
     module: str
     filename: Optional[str]
     inc: Optional[List[str]]
+    family = "perl-module"
 
     def __init__(self, module, filename=None, inc=None):
-        super(PerlModuleRequirement, self).__init__("perl-module")
         self.module = module
         self.filename = filename
         self.inc = inc
@@ -182,10 +255,10 @@ class PerlModuleRequirement(Requirement):
 class VagueDependencyRequirement(Requirement):
 
     name: str
+    family = "vague"
     minimum_version: Optional[str] = None
 
     def __init__(self, name, minimum_version=None):
-        super(VagueDependencyRequirement, self).__init__("vague")
         self.name = name
         self.minimum_version = minimum_version
 
@@ -193,28 +266,29 @@ class VagueDependencyRequirement(Requirement):
         if " " not in self.name:
             yield BinaryRequirement(self.name)
             yield LibraryRequirement(self.name)
-            yield PkgConfigRequirement(self.name, minimum_version=self.minimum_version)
+            yield PkgConfigRequirement(
+                self.name, minimum_version=self.minimum_version)
             if self.name.lower() != self.name:
                 yield BinaryRequirement(self.name.lower())
                 yield LibraryRequirement(self.name.lower())
-                yield PkgConfigRequirement(self.name.lower(), minimum_version=self.minimum_version)
+                yield PkgConfigRequirement(
+                    self.name.lower(), minimum_version=self.minimum_version)
             try:
                 from .resolver.apt import AptRequirement
             except ModuleNotFoundError:
                 pass
             else:
-                yield AptRequirement.simple(self.name.lower(), minimum_version=self.minimum_version)
+                yield AptRequirement.simple(
+                    self.name.lower(), minimum_version=self.minimum_version)
                 if self.name.lower().startswith('lib'):
                     devname = '%s-dev' % self.name.lower()
                 else:
                     devname = 'lib%s-dev' % self.name.lower()
-                yield AptRequirement.simple(devname, minimum_version=self.minimum_version)
+                yield AptRequirement.simple(
+                    devname, minimum_version=self.minimum_version)
 
     def met(self, session):
-        for x in self.expand():
-            if x.met(session):
-                return True
-        return False
+        return any(x.met(session) for x in self.expand())
 
     def __repr__(self):
         return "%s(%r)" % (type(self).__name__, self.name)
@@ -228,9 +302,9 @@ class VagueDependencyRequirement(Requirement):
 class NodePackageRequirement(Requirement):
 
     package: str
+    family = "npm-package"
 
     def __init__(self, package):
-        super(NodePackageRequirement, self).__init__("npm-package")
         self.package = package
 
     def __repr__(self):
@@ -240,9 +314,9 @@ class NodePackageRequirement(Requirement):
 class LuaModuleRequirement(Requirement):
 
     module: str
+    family = "lua-module"
 
     def __init__(self, module):
-        super(LuaModuleRequirement, self).__init__("lua-module")
         self.module = module
 
     def __repr__(self):
@@ -271,8 +345,9 @@ class PerlPreDeclaredRequirement(Requirement):
         'auto_set_bugtracker': 'Module::Install::Bugtracker',
         }
 
+    family = "perl-predeclared"
+
     def __init__(self, name):
-        super(PerlPreDeclaredRequirement, self).__init__("perl-predeclared")
         self.name = name
 
     def lookup_module(self):
@@ -286,9 +361,9 @@ class PerlPreDeclaredRequirement(Requirement):
 class NodeModuleRequirement(Requirement):
 
     module: str
+    family = "npm-module"
 
     def __init__(self, module):
-        super(NodeModuleRequirement, self).__init__("npm-module")
         self.module = module
 
     def __repr__(self):
@@ -301,9 +376,10 @@ class CargoCrateRequirement(Requirement):
     features: Set[str]
     api_version: Optional[str]
     minimum_version: Optional[str]
+    family = "cargo-crate"
 
-    def __init__(self, crate, features=None, api_version=None, minimum_version=None):
-        super(CargoCrateRequirement, self).__init__("cargo-crate")
+    def __init__(self, crate, features=None, api_version=None,
+                 minimum_version=None):
         self.crate = crate
         if features is None:
             features = set()
@@ -334,9 +410,9 @@ class CargoCrateRequirement(Requirement):
 class PkgConfigRequirement(Requirement):
 
     module: str
+    family = "pkg-config"
 
     def __init__(self, module, minimum_version=None):
-        super(PkgConfigRequirement, self).__init__("pkg-config")
         self.module = module
         self.minimum_version = minimum_version
 
@@ -348,9 +424,9 @@ class PkgConfigRequirement(Requirement):
 class PathRequirement(Requirement):
 
     path: str
+    family = "path"
 
     def __init__(self, path):
-        super(PathRequirement, self).__init__("path")
         self.path = path
 
     def __repr__(self):
@@ -360,9 +436,9 @@ class PathRequirement(Requirement):
 class CHeaderRequirement(Requirement):
 
     header: str
+    family = "c-header"
 
     def __init__(self, header):
-        super(CHeaderRequirement, self).__init__("c-header")
         self.header = header
 
     def __repr__(self):
@@ -370,16 +446,15 @@ class CHeaderRequirement(Requirement):
 
 
 class JavaScriptRuntimeRequirement(Requirement):
-    def __init__(self):
-        super(JavaScriptRuntimeRequirement, self).__init__("javascript-runtime")
+    family = "javascript-runtime"
 
 
 class ValaPackageRequirement(Requirement):
 
     package: str
+    family = "vala"
 
     def __init__(self, package: str):
-        super(ValaPackageRequirement, self).__init__("vala")
         self.package = package
 
 
@@ -387,9 +462,9 @@ class RubyGemRequirement(Requirement):
 
     gem: str
     minimum_version: Optional[str]
+    family = "gem"
 
     def __init__(self, gem: str, minimum_version: Optional[str]):
-        super(RubyGemRequirement, self).__init__("gem")
         self.gem = gem
         self.minimum_version = minimum_version
 
@@ -398,9 +473,9 @@ class GoPackageRequirement(Requirement):
 
     package: str
     version: Optional[str]
+    family = "go-package"
 
     def __init__(self, package: str, version: Optional[str] = None):
-        super(GoPackageRequirement, self).__init__("go-package")
         self.package = package
         self.version = version
 
@@ -417,9 +492,9 @@ class GoPackageRequirement(Requirement):
 class GoRequirement(Requirement):
 
     version: Optional[str]
+    family = "go"
 
     def __init__(self, version: Optional[str] = None):
-        super(GoRequirement, self).__init__("go")
         self.version = version
 
     def __str__(self):
@@ -429,18 +504,18 @@ class GoRequirement(Requirement):
 class DhAddonRequirement(Requirement):
 
     path: str
+    family = "dh-addon"
 
     def __init__(self, path: str):
-        super(DhAddonRequirement, self).__init__("dh-addon")
         self.path = path
 
 
 class PhpClassRequirement(Requirement):
 
     php_class: str
+    family = "php-class"
 
     def __init__(self, php_class: str):
-        super(PhpClassRequirement, self).__init__("php-class")
         self.php_class = php_class
 
 
@@ -448,9 +523,9 @@ class RPackageRequirement(Requirement):
 
     package: str
     minimum_version: Optional[str]
+    family = "r-package"
 
     def __init__(self, package: str, minimum_version: Optional[str] = None):
-        super(RPackageRequirement, self).__init__("r-package")
         self.package = package
         self.minimum_version = minimum_version
 
@@ -463,7 +538,8 @@ class RPackageRequirement(Requirement):
 
     def __str__(self):
         if self.minimum_version:
-            return "R package: %s (>= %s)" % (self.package, self.minimum_version)
+            return "R package: %s (>= %s)" % (
+                self.package, self.minimum_version)
         else:
             return "R package: %s" % (self.package,)
 
@@ -483,9 +559,9 @@ class OctavePackageRequirement(Requirement):
 
     package: str
     minimum_version: Optional[str]
+    family = "octave-package"
 
     def __init__(self, package: str, minimum_version: Optional[str] = None):
-        super(OctavePackageRequirement, self).__init__("octave-package")
         self.package = package
         self.minimum_version = minimum_version
 
@@ -498,7 +574,8 @@ class OctavePackageRequirement(Requirement):
 
     def __str__(self):
         if self.minimum_version:
-            return "Octave package: %s (>= %s)" % (self.package, self.minimum_version)
+            return "Octave package: %s (>= %s)" % (
+                self.package, self.minimum_version)
         else:
             return "Octave package: %s" % (self.package,)
 
@@ -517,9 +594,9 @@ class OctavePackageRequirement(Requirement):
 class LibraryRequirement(Requirement):
 
     library: str
+    family = "lib"
 
     def __init__(self, library: str):
-        super(LibraryRequirement, self).__init__("lib")
         self.library = library
 
 
@@ -527,9 +604,9 @@ class StaticLibraryRequirement(Requirement):
 
     library: str
     filename: str
+    family = "static-lib"
 
     def __init__(self, library: str, filename: str):
-        super(StaticLibraryRequirement, self).__init__("static-lib")
         self.library = library
         self.filename = filename
 
@@ -537,18 +614,18 @@ class StaticLibraryRequirement(Requirement):
 class RubyFileRequirement(Requirement):
 
     filename: str
+    family = "ruby-file"
 
     def __init__(self, filename: str):
-        super(RubyFileRequirement, self).__init__("ruby-file")
         self.filename = filename
 
 
 class XmlEntityRequirement(Requirement):
 
     url: str
+    family = "xml-entity"
 
     def __init__(self, url: str):
-        super(XmlEntityRequirement, self).__init__("xml-entity")
         self.url = url
 
 
@@ -556,9 +633,9 @@ class SprocketsFileRequirement(Requirement):
 
     content_type: str
     name: str
+    family = "sprockets-file"
 
     def __init__(self, content_type: str, name: str):
-        super(SprocketsFileRequirement, self).__init__("sprockets-file")
         self.content_type = content_type
         self.name = name
 
@@ -566,9 +643,9 @@ class SprocketsFileRequirement(Requirement):
 class JavaClassRequirement(Requirement):
 
     classname: str
+    family = "java-class"
 
     def __init__(self, classname: str):
-        super(JavaClassRequirement, self).__init__("java-class")
         self.classname = classname
 
 
@@ -576,9 +653,9 @@ class CMakefileRequirement(Requirement):
 
     filename: str
     version: Optional[str]
+    family = "cmake-file"
 
     def __init__(self, filename: str, version=None):
-        super(CMakefileRequirement, self).__init__("cmake-file")
         self.filename = filename
         self.version = version
 
@@ -586,9 +663,9 @@ class CMakefileRequirement(Requirement):
 class HaskellPackageRequirement(Requirement):
 
     package: str
+    family = "haskell-package"
 
     def __init__(self, package: str, specs=None):
-        super(HaskellPackageRequirement, self).__init__("haskell-package")
         self.package = package
         self.specs = specs
 
@@ -604,9 +681,9 @@ class MavenArtifactRequirement(Requirement):
     artifact_id: str
     version: Optional[str]
     kind: Optional[str]
+    family = "maven-artifact"
 
     def __init__(self, group_id, artifact_id, version=None, kind=None):
-        super(MavenArtifactRequirement, self).__init__("maven-artifact")
         self.group_id = group_id
         self.artifact_id = artifact_id
         self.version = version
@@ -645,17 +722,16 @@ class MavenArtifactRequirement(Requirement):
 
 
 class GnomeCommonRequirement(Requirement):
-    def __init__(self):
-        super(GnomeCommonRequirement, self).__init__("gnome-common")
+    family = "gnome-common"
 
 
 class JDKFileRequirement(Requirement):
 
     jdk_path: str
     filename: str
+    family = "jdk-file"
 
     def __init__(self, jdk_path: str, filename: str):
-        super(JDKFileRequirement, self).__init__("jdk-file")
         self.jdk_path = jdk_path
         self.filename = filename
 
@@ -665,55 +741,70 @@ class JDKFileRequirement(Requirement):
 
 
 class JDKRequirement(Requirement):
-    def __init__(self):
-        super(JDKRequirement, self).__init__("jdk")
+    family = "jdk"
 
 
 class JRERequirement(Requirement):
-    def __init__(self):
-        super(JRERequirement, self).__init__("jre")
+    family = "jre"
+
+
+class QtModuleRequirement(Requirement):
+    family = "qt-module"
+
+    def __init__(self, module):
+        self.module = module
 
 
 class QTRequirement(Requirement):
-    def __init__(self):
-        super(QTRequirement, self).__init__("qt")
+    family = "qt"
 
 
 class X11Requirement(Requirement):
-    def __init__(self):
-        super(X11Requirement, self).__init__("x11")
+    family = "x11"
 
 
 class CertificateAuthorityRequirement(Requirement):
+    family = "ca-cert"
+
     def __init__(self, url):
-        super(CertificateAuthorityRequirement, self).__init__("ca-cert")
         self.url = url
 
 
 class PerlFileRequirement(Requirement):
 
     filename: str
+    family = "perl-file"
 
     def __init__(self, filename: str):
-        super(PerlFileRequirement, self).__init__("perl-file")
         self.filename = filename
 
 
 class AutoconfMacroRequirement(Requirement):
 
+    family = "autoconf-macro"
     macro: str
 
     def __init__(self, macro: str):
-        super(AutoconfMacroRequirement, self).__init__("autoconf-macro")
         self.macro = macro
 
+    def _json(self):
+        return self.macro
+
+    @classmethod
+    def _from_json(cls, macro):
+        return cls(macro)
+
+
+Requirement.register_json(AutoconfMacroRequirement)
+
 
 class LibtoolRequirement(Requirement):
-    def __init__(self):
-        super(LibtoolRequirement, self).__init__("libtool")
+    family = "libtool"
 
 
 class IntrospectionTypelibRequirement(Requirement):
+    family = "introspection-type-lib"
+
     def __init__(self, library):
         self.library = library
 
@@ -723,9 +814,9 @@ class PythonModuleRequirement(Requirement):
     module: str
     python_version: Optional[str]
     minimum_version: Optional[str]
+    family = "python-module"
 
     def __init__(self, module, python_version=None, minimum_version=None):
-        super(PythonModuleRequirement, self).__init__("python-module")
         self.module = module
         self.python_version = python_version
         self.minimum_version = minimum_version
@@ -760,25 +851,25 @@ class PythonModuleRequirement(Requirement):
 class BoostComponentRequirement(Requirement):
 
     name: str
+    family = "boost-component"
 
     def __init__(self, name):
-        super(BoostComponentRequirement, self).__init__("boost-component")
         self.name = name
 
 
 class KF5ComponentRequirement(Requirement):
 
     name: str
+    family = "kf5-component"
 
     def __init__(self, name):
-        super(KF5ComponentRequirement, self).__init__("kf5-component")
         self.name = name
 
 
 class GnulibDirectoryRequirement(Requirement):
 
     directory: str
+    family = "gnulib"
 
     def __init__(self, directory):
-        super(GnulibDirectoryRequirement, self).__init__("gnulib")
         self.directory = directory
