@@ -18,9 +18,9 @@
 
 import logging
 import subprocess
-from typing import Optional, List, Type
+from typing import Optional, List, Type, Iterator, Dict
 
-from .. import UnidentifiedError
+from .. import UnidentifiedError, Requirement
 from ..fix_build import run_detecting_problems
 from ..session import Session
 
@@ -30,23 +30,26 @@ class UnsatisfiedRequirements(Exception):
         self.requirements = reqs
 
 
-class Resolver(object):
+class Resolver:
 
     name: str
 
-    def __init__(self, session, user_local):
-        raise NotImplementedError(self.__init__)
+    def __init__(self, session: Session, user_local: bool):
+        raise NotImplementedError
 
-    def install(self, requirements):
+    def install(self, requirements: List[Requirement]) -> None:
         raise NotImplementedError(self.install)
 
-    def resolve(self, requirement):
+    def resolve(self, requirement: Requirement) -> Optional[Requirement]:
         raise NotImplementedError(self.resolve)
 
-    def explain(self, requirements):
+    def resolve_all(self, requirement: Requirement) -> List[Requirement]:
+        raise NotImplementedError(self.resolve_all)
+
+    def explain(self, requirements: List[Requirement]) -> Iterator[List[str]]:
         raise NotImplementedError(self.explain)
 
-    def env(self):
+    def env(self) -> Dict[str, str]:
         return {}
 
 
@@ -120,7 +123,8 @@ class TlmgrResolver(Resolver):
         self.repository = repository
 
     def __str__(self):
-        if self.repository.startswith('http://') or self.repository.startswith('https://'):
+        if (self.repository.startswith('http://')
+                or self.repository.startswith('https://')):
             return 'tlmgr(%r)' % self.repository
         else:
             return self.repository
@@ -165,7 +169,8 @@ class TlmgrResolver(Resolver):
             try:
                 run_detecting_problems(self.session, cmd, user=user)
             except UnidentifiedError as e:
-                if "tlmgr: user mode not initialized, please read the documentation!" in e.lines:
+                if ("tlmgr: user mode not initialized, "
+                        "please read the documentation!") in e.lines:
                     self.session.check_call(['tlmgr', 'init-usertree'])
                 else:
                     raise
@@ -297,7 +302,8 @@ class BioconductorResolver(RResolver):
 
     def __init__(self, session, user_local=False):
         super(BioconductorResolver, self).__init__(
-            session, "https://hedgehog.fhcrc.org/bioconductor", user_local=user_local
+            session, "https://hedgehog.fhcrc.org/bioconductor",
+            user_local=user_local
         )
 
 
@@ -319,7 +325,8 @@ class HackageResolver(Resolver):
         extra_args = []
         if self.user_local:
             extra_args.append("--user")
-        return ["cabal", "install"] + extra_args + [req.package for req in reqs]
+        return (["cabal", "install"] + extra_args
+                + [req.package for req in reqs])
 
     def install(self, requirements):
         from ..requirements import HaskellPackageRequirement
@@ -460,7 +467,7 @@ NPM_COMMAND_PACKAGES = {
     "standard": "standard",
     "jshint": "jshint",
     "if-node-version": "if-node-version",
-    "babel": "babel",
+    "babel-cli": "babel",
     "c8": "c8",
     "prettier-standard": "prettier-standard",
 }
@@ -480,13 +487,32 @@ class NpmResolver(Resolver):
     def __repr__(self):
         return "%s(%r)" % (type(self).__name__, self.session)
 
-    def install(self, requirements):
+    @classmethod
+    def _to_node_package_req(cls, requirement):
         from ..requirements import (
             NodePackageRequirement,
             NodeModuleRequirement,
             BinaryRequirement,
         )
+        if isinstance(requirement, BinaryRequirement):
+            try:
+                package = NPM_COMMAND_PACKAGES[requirement.binary_name]
+            except KeyError:
+                pass
+            else:
+                return NodePackageRequirement(package)
+        if isinstance(requirement, NodeModuleRequirement):
+            # TODO: Is this legit?
+            parts = requirement.module.split("/")
+            if parts[0].startswith('@'):
+                return NodePackageRequirement('/'.join(parts[:2]))
+            else:
+                return NodePackageRequirement(parts[0])
+        if isinstance(requirement, NodePackageRequirement):
+            return requirement
+        return None
 
+    def install(self, requirements):
         if self.user_local:
             user = None
         else:
@@ -494,46 +520,27 @@ class NpmResolver(Resolver):
 
         missing = []
         for requirement in requirements:
-            if isinstance(requirement, BinaryRequirement):
-                try:
-                    package = NPM_COMMAND_PACKAGES[requirement.binary_name]
-                except KeyError:
-                    pass
-                else:
-                    requirement = NodePackageRequirement(package)
-            if isinstance(requirement, NodeModuleRequirement):
-                # TODO: Is this legit?
-                parts = requirement.module.split("/")
-                if parts[0].startswith('@'):
-                    requirement = NodePackageRequirement('/'.join(parts[:2]))
-                else:
-                    requirement = NodePackageRequirement(parts[0])
-            if not isinstance(requirement, NodePackageRequirement):
+            node_pkg_requirement = self._to_node_package_req(requirement)
+            if node_pkg_requirement is None:
                 missing.append(requirement)
                 continue
             cmd = ["npm", "install"]
             if not self.user_local:
                 cmd.append('-g')
-            cmd.append(requirement.package)
+            cmd.append(node_pkg_requirement.package)
             logging.info("npm: running %r", cmd)
             run_detecting_problems(self.session, cmd, user=user)
         if missing:
             raise UnsatisfiedRequirements(missing)
 
     def explain(self, requirements):
-        from ..requirements import NodePackageRequirement
-
         nodereqs = []
         packages = []
         for requirement in requirements:
-            if not isinstance(requirement, NodePackageRequirement):
-                continue
-            try:
-                package = NPM_COMMAND_PACKAGES[requirement.command]
-            except KeyError:
-                continue
-            nodereqs.append(requirement)
-            packages.append(package)
+            node_pkg_requirement = self._to_node_package_req(requirement)
+            if node_pkg_requirement is not None:
+                packages.append(node_pkg_requirement.package)
+                nodereqs.append(requirement)
         if nodereqs:
             yield (["npm", "-g", "install"] + packages, nodereqs)
 
@@ -585,10 +592,12 @@ NATIVE_RESOLVER_CLS: List[Type[Resolver]] = [
 
 
 def native_resolvers(session, user_local):
-    return StackedResolver([kls(session, user_local) for kls in NATIVE_RESOLVER_CLS])
+    return StackedResolver(
+        [kls(session, user_local) for kls in NATIVE_RESOLVER_CLS])
 
 
-def select_resolvers(session, user_local, resolvers) -> Optional[Resolver]:
+def select_resolvers(session, user_local, resolvers,
+                     dep_server_url=None) -> Optional[Resolver]:
     selected = []
     for resolver in resolvers:
         for kls in NATIVE_RESOLVER_CLS:
@@ -597,12 +606,19 @@ def select_resolvers(session, user_local, resolvers) -> Optional[Resolver]:
                 break
         else:
             if resolver == 'native':
-                selected.extend([kls(session, user_local) for kls in NATIVE_RESOLVER_CLS])
+                selected.extend([
+                    kls(session, user_local) for kls in NATIVE_RESOLVER_CLS])
             elif resolver == 'apt':
                 if user_local:
-                    raise NotImplementedError('user local not supported for apt')
-                from .apt import AptResolver
-                selected.append(AptResolver.from_session(session))
+                    raise NotImplementedError(
+                        'user local not supported for apt')
+                if dep_server_url:
+                    from .dep_server import DepServerAptResolver
+                    selected.append(DepServerAptResolver.from_session(
+                        session, dep_server_url))
+                else:
+                    from .apt import AptResolver
+                    selected.append(AptResolver.from_session(session))
             else:
                 raise KeyError(resolver)
     if len(selected) == 0:
@@ -612,7 +628,9 @@ def select_resolvers(session, user_local, resolvers) -> Optional[Resolver]:
     return StackedResolver(selected)
 
 
-def auto_resolver(session: Session, explain: bool = False, system_wide: Optional[bool] = None):
+def auto_resolver(session: Session, explain: bool = False,
+                  system_wide: Optional[bool] = None,
+                  dep_server_url: Optional[str] = None):
     # if session is SchrootSession or if we're root, use apt
     from ..session.schroot import SchrootSession
     from ..session import get_user
@@ -632,6 +650,12 @@ def auto_resolver(session: Session, explain: bool = False, system_wide: Optional
         except ModuleNotFoundError:
             pass
         else:
-            resolvers.append(AptResolver.from_session(session))
-    resolvers.extend([kls(session, not system_wide) for kls in NATIVE_RESOLVER_CLS])
+            if dep_server_url:
+                from .dep_server import DepServerAptResolver
+                resolvers.append(
+                    DepServerAptResolver.from_session(session, dep_server_url))
+            else:
+                resolvers.append(AptResolver.from_session(session))
+    resolvers.extend([kls(session, not system_wide)
+                      for kls in NATIVE_RESOLVER_CLS])
     return StackedResolver(resolvers)
