@@ -66,7 +66,8 @@ class SchrootSession(Session):
                 if line.startswith(b"E: "):
                     logging.error("%s", line[3:].decode(errors="replace"))
             logging.warning(
-                "Failed to close schroot session %s, leaving stray.", self.session_id
+                "Failed to close schroot session %s, leaving stray.",
+                self.session_id
             )
             self.session_id = None
             return False
@@ -80,19 +81,23 @@ class SchrootSession(Session):
         stderr = tempfile.TemporaryFile()
         try:
             self.session_id = (
-                subprocess.check_output(["schroot", "-c", self.chroot, "-b"], stderr=stderr)
+                subprocess.check_output(
+                    ["schroot", "-c", self.chroot, "-b"], stderr=stderr)
                 .strip()
                 .decode()
             )
-        except subprocess.CalledProcessError:
+        except subprocess.CalledProcessError as e:
             stderr.seek(0)
             errlines = stderr.readlines()
             if len(errlines) == 1:
                 raise SessionSetupFailure(
-                    errlines[0].rstrip().decode(), errlines=errlines)
+                    errlines[0].rstrip().decode(), errlines=errlines) from e
+            elif len(errlines) == 0:
+                raise SessionSetupFailure(
+                    "No output from schroot", errlines=errlines) from e
             else:
                 raise SessionSetupFailure(
-                    errlines[-1].decode(), errlines=errlines)
+                    errlines[-1].decode(), errlines=errlines) from e
         logging.info(
             "Opened schroot session %s (from %s)", self.session_id, self.chroot
         )
@@ -154,7 +159,7 @@ class SchrootSession(Session):
                 self._run_argv(argv, cwd, user, env=env), close_fds=close_fds
             )
         except subprocess.CalledProcessError as e:
-            raise subprocess.CalledProcessError(e.returncode, argv)
+            raise subprocess.CalledProcessError(e.returncode, argv) from e
 
     def check_output(
         self,
@@ -164,24 +169,28 @@ class SchrootSession(Session):
         env: Optional[Dict[str, str]] = None,
     ) -> bytes:
         try:
-            return subprocess.check_output(self._run_argv(argv, cwd, user, env=env))
+            return subprocess.check_output(
+                self._run_argv(argv, cwd, user, env=env))
         except subprocess.CalledProcessError as e:
-            raise subprocess.CalledProcessError(e.returncode, argv)
+            raise subprocess.CalledProcessError(e.returncode, argv) from e
 
     def Popen(
-        self, argv, cwd: Optional[str] = None, user: Optional[str] = None, **kwargs
+        self, argv, cwd: Optional[str] = None, user: Optional[str] = None,
+        **kwargs
     ):
         return subprocess.Popen(self._run_argv(argv, cwd, user), **kwargs)
 
     def call(
-        self, argv: List[str], cwd: Optional[str] = None, user: Optional[str] = None
+        self, argv: List[str], cwd: Optional[str] = None,
+        user: Optional[str] = None
     ):
         return subprocess.call(self._run_argv(argv, cwd, user))
 
     def create_home(self) -> None:
         """Create the user's home directory."""
         home = (
-            self.check_output(["sh", "-c", "echo $HOME"], cwd="/").decode().rstrip("\n")
+            self.check_output(
+                ["sh", "-c", "echo $HOME"], cwd="/").decode().rstrip("\n")
         )
         user = (
             self.check_output(["sh", "-c", "echo $LOGNAME"], cwd="/")
@@ -197,7 +206,8 @@ class SchrootSession(Session):
             return os.path.join(self.location, path.lstrip("/"))
         if self._cwd is None:
             raise ValueError("no cwd set")
-        return os.path.join(self.location, os.path.join(self._cwd, path).lstrip("/"))
+        return os.path.join(
+            self.location, os.path.join(self._cwd, path).lstrip("/"))
 
     def exists(self, path: str) -> bool:
         fullpath = self.external_path(path)
